<?php
add_filter( 'sunshine_options_galleries', 'sunshine_watermark_options', 45 );
function sunshine_watermark_options( $options ) {
	$options[] = array( 'name' => __( 'Watermark','sunshine' ), 'type' => 'title', 'desc' => __( 'Add image to all files uploaded to Sunshine (except digital download files)','sunshine' ) );
/*
	$attachments = get_posts( array( 'post_type' => 'attachment', 'post_mime_type' => 'image/png', 'post_parent' => 0, 'posts_per_page' => -1 ) );
	$media[0] = __( 'No image', 'sunshine' );
	foreach ( $attachments as $attachment ) {
		$media[$attachment->ID] = $attachment->post_title;
	}
	*/
	$options[] = array(
		'name' => __( 'Image', 'sunshine' ),
		'id'   => 'watermark_image',
		'type' => 'image',
		'desc' => __( 'Transparent PNG works best','sunshine' )
	);
	$options[] = array(
		'name' => __( 'Max Size', 'sunshine' ),
		'id'   => 'watermark_max_size',
		'type' => 'number',
		'desc' => __( 'From 1-100, a max percent size of the image', 'sunshine' ),
		'restrict' => array(
			'min' => 1,
			'max' => 100
		)
	);
	$options[] = array(
		'name' => __( 'Position', 'sunshine' ),
		'id'   => 'watermark_position',
		'type' => 'select',
		'options' => array(
			'topleft' => __( 'Top Left','sunshine' ),
			'topright' => __( 'Top Right','sunshine' ),
			'bottomleft' => __( 'Bottom Left','sunshine' ),
			'bottomright' => __( 'Bottom Right','sunshine' ),
			'center' => __( 'Center','sunshine' ),
			'repeat' => __( 'Repeat','sunshine' )
		)
	);
	$options[] = array(
		'name' => __( 'Margin from edge', 'sunshine' ),
		'desc' => __( 'Not used for "Center" position', 'sunshine' ),
		'id'   => 'watermark_margin',
		'type' => 'number',
		'css' => 'width: 50px;'
	);
	$options[] = array(
		'name' => __( 'Watermark Thumbnails', 'sunshine' ),
		'id'   => 'watermark_thumbnail',
		'type' => 'checkbox',
		'options' => array( 1 )
	);


	return $options;
}

function sunshine_watermark_image( $attachment_id, $metadata = array(), $passed_image_size = '' ) {
	global $sunshine;
	$attachment = get_post( $attachment_id );
	if ( get_post_type( $attachment->post_parent ) == 'sunshine-gallery' && $sunshine->options['watermark_image'] && wp_attachment_is_image( $attachment_id ) ) {

		$watermark_image = get_attached_file( $sunshine->options['watermark_image'] );
		$watermark_file_type = wp_check_filetype( $watermark_image );

		if ( file_exists( $watermark_image ) && $watermark_file_type['ext'] == 'png' ) {

			$image = get_attached_file( $attachment_id, 'full' );
			if ( !empty( $passed_image_size ) ) {
				$image_size = $passed_image_size;
			}
			if ( empty( $image_size ) ) {
				$image_size = apply_filters( 'sunshine_image_size', 'full' );
			}
			if ( empty( $metadata ) ) {
				$metadata = wp_get_attachment_metadata( $attachment_id );
			}
			$image_basename = basename( $image );
			$image_path = str_replace( $image_basename, '', $image );
			if ( $image_size != 'full' && !empty( $image ) && !empty( $metadata ) && !empty( $metadata['sizes'][ $image_size ]['file'] ) ) {
				$image = $image_path . $metadata['sizes'][ $image_size ]['file'];
			}

			if ( !file_exists( $image ) ) {
				return;
			}

			$watermark = imagecreatefrompng( $watermark_image );
			$new_image = imagecreatefromjpeg( $image );

			$margin = ( $sunshine->options['watermark_margin'] != '' ) ? $sunshine->options['watermark_margin'] : 30;
			$watermark_width = imagesx( $watermark );
			$watermark_height = imagesy( $watermark );
			$new_image_width = imagesx( $new_image );
			$new_image_height = imagesy( $new_image );

			$resize_watermark = false;
			$watermark_ratio = $watermark_width / $watermark_height;

			// Make watermark size no larger than image being applied to
			if ( $watermark_width > $new_image_width ) {
				$resize_watermark = true;
				$new_watermark_width = $new_image_width;
				$new_watermark_height = $new_watermark_width / $watermark_ratio;
			}

			if ( !empty( $sunshine->options['watermark_max_size'] ) && $sunshine->options['watermark_max_size'] > 0 ) {
				$max_watermark_size_percent = $sunshine->options['watermark_max_size'] / 100;
				$new_watermark_width = $watermark_width;
				$new_watermark_height = $watermark_height;
				if ( ( $new_image_width * $max_watermark_size_percent ) < $new_watermark_width ) {
					$resize_watermark = true;
					$new_watermark_width = $new_image_width * $max_watermark_size_percent;
					$new_watermark_height = $new_watermark_width / $watermark_ratio;
				}
			}

			if ( $resize_watermark ) {
				$new_watermark = imagecreatetruecolor( $new_watermark_width, $new_watermark_height );
				imagealphablending( $new_watermark, false );
				imagesavealpha( $new_watermark, true );
				imagecopyresampled( $new_watermark, $watermark, 0, 0, 0, 0, $new_watermark_width, $new_watermark_height, $watermark_width, $watermark_height );
				$watermark = $new_watermark;
				$watermark_width = $new_watermark_width;
				$watermark_height = $new_watermark_height;
			}


			if ( $sunshine->options['watermark_position'] == 'topleft' ) {
				$x_pos = $margin;
				$y_pos = $margin;
			} elseif ( $sunshine->options['watermark_position'] == 'topright' ) {
				$x_pos = $new_image_width - $watermark_width - $margin;
				$y_pos = $margin;
			} elseif ( $sunshine->options['watermark_position'] == 'bottomleft' ) {
				$x_pos = $margin;
				$y_pos = $new_image_height - $watermark_height - $margin;
			} elseif ( $sunshine->options['watermark_position'] == 'bottomright' ) {
				$x_pos = $new_image_width - $watermark_width - $margin;
				$y_pos = $new_image_height - $watermark_height - $margin;
			} else {
				$x_pos = ( $new_image_width/2 ) - ( $watermark_width/2 );
				$y_pos = ( $new_image_height/2 ) - ( $watermark_height/2 );
			}

			if ( $sunshine->options['watermark_position'] == 'repeat' ) {
				imagesettile( $new_image, $watermark );
				imagefilledrectangle( $new_image, 0, 0, $new_image_width, $new_image_height, IMG_COLOR_TILED );
			} else {
				imagecopy( $new_image, $watermark, $x_pos, $y_pos, 0, 0, $watermark_width, $watermark_height );
			}

			// Output and free memory
			imagejpeg( $new_image, $image, 100 );
			imagedestroy( $new_image );

			// Watermark the thumbnail if needed
			if ( $sunshine->options['watermark_thumbnail'] && empty( $passed_image_size ) ) {
				$image_editor = wp_get_image_editor( $image );
				$image_editor->resize( $sunshine->options['thumbnail_width'], $sunshine->options['thumbnail_height'], $sunshine->options['thumbnail_crop'] );
				$thumb_path = '';
				if ( isset( $metadata['sizes']['sunshine-thumbnail']['file'] ) ) {
					$thumb_path = $image_path . $metadata['sizes']['sunshine-thumbnail']['file'];
				}
				$image_editor->save( $thumb_path );
			}
		}
	}
}

// Add the watermark
add_filter( 'wp_generate_attachment_metadata', 'sunshine_watermark_media_upload', 5, 2 );
function sunshine_watermark_media_upload( $metadata, $attachment_id ) {
	sunshine_watermark_image( $attachment_id, $metadata );
	return $metadata;
}

?>
