<?php
require_once( 'sunshine-menu.php' );
require_once( 'sunshine-dashboard.php' );
require_once( 'sunshine-notices.php' );
require_once( 'sunshine-galleries.php' );
require_once( 'sunshine-image-processor.php' );
require_once( 'sunshine-products.php' );
require_once( 'sunshine-bulk-add-products.php' );
require_once( 'sunshine-orders.php' );
require_once( 'sunshine-users.php' );

add_filter(
	'jpeg_quality',
	function( $arg ) {
		return 100;
	},
	999
);

add_filter( 'wp_image_editors', 'sunshine_force_imagick' );
function sunshine_force_imagick( $editors ) {
	if ( extension_loaded( 'imagick' ) ) {
		$editors = array( 'WP_Image_Editor_Imagick' );
	}
	return $editors;
}

if ( ( isset( $_GET['page'] ) && $_GET['page'] == 'sunshine' ) || ( isset( $_POST['currentTab'] ) && isset( $_POST['action'] ) && $_POST['action'] == 'update' ) ) {
	include_once( SUNSHINE_PATH . 'classes/sf-class-settings.php' );
	$sunshine_options = new SF_Settings_API( 'sunshine', 'Sunshine Settings', 'admin.php', __FILE__ );
	$sunshine_options->load_options( SUNSHINE_PATH . '/sunshine-options.php' );
}

if ( isset( $_GET['page'] ) && $_GET['page'] == 'sunshine' ) {
	flush_rewrite_rules();
}

add_action( 'admin_init', 'sunshine_install_redirect' );
function sunshine_install_redirect() {
	if ( get_option( 'sunshine_install_redirect', false ) ) {
		delete_option( 'sunshine_install_redirect' );
		wp_redirect( admin_url( '/admin.php?page=sunshine_about&sunshine_install' ) );
		exit;
	}
}

function sunshine_manual_update() {
	echo '<div id="message" class="updated"><p>' . __( 'Update completed', 'sunshine' ) . '</p></div>';
}

add_action( 'admin_enqueue_scripts', 'sunshine_admin_cssjs', 5 );
function sunshine_admin_cssjs() {
	wp_register_style( 'sunshine-admin-css', SUNSHINE_URL . 'assets/css/admin.css', '', SUNSHINE_VERSION );
	wp_enqueue_style( 'sunshine-admin-css' );
	wp_register_style( 'sunshine-jquery-ui', SUNSHINE_URL . 'assets/jqueryui/smoothness/jquery-ui-1.9.2.custom.css', '', SUNSHINE_VERSION );
	wp_register_script( 'select2', SUNSHINE_URL . 'assets/js/select2/select2.min.js', array( 'jquery' ), '4.0.13' );
	wp_register_style( 'select2', SUNSHINE_URL . 'assets/js/select2/select2.min.css', '4.0.13' );
	if ( isset( $_GET['page'] ) && $_GET['page'] == 'sunshine' ) {
		wp_enqueue_style( 'wp-color-picker' );
		wp_enqueue_script( 'wp-color-picker' );
		wp_enqueue_style( 'select2' );
		wp_enqueue_script( 'select2' );
	}
}


function sunshine_about() {
	global $sunshine;
	?>
	<div id="fb-root"></div>
	<script>(function(d, s, id) {
		var js, fjs = d.getElementsByTagName(s)[0];
		if (d.getElementById(id)) return;
		js = d.createElement(s); js.id = id;
		js.src = "//connect.facebook.net/en_US/sdk.js#xfbml=1&version=v2.4&appId=228213277229357";
		fjs.parentNode.insertBefore(js, fjs);
	}(document, 'script', 'facebook-jssdk'));</script>

	<div id="sunshine-header">
		<h1><?php printf( __( 'Welcome to Sunshine Photo Cart %s', 'sunshine' ), $sunshine->version ); ?></h1>
		<p>
			<?php
			if ( isset( $_GET['sunshine_updated'] ) ) {
				$message = __( 'Thank you for updating to the latest version!', 'sunshine' );
			} else {
				$message = __( 'Thank you for installing!', 'sunshine' );
			}
			printf( __( '<strong>%1$s</strong> Sunshine %2$s is the most comprehensive client proofing and photo cart plugin for WordPress. We hope you enjoy greater selling success!', 'sunshine' ), $message, $sunshine->version );
			?>
		</p>
	</div>

	<div class="sunshine-wrap">

		<?php if ( ! $sunshine->is_pro() ) { ?>
			<div class="sunshine-upgrade">
				<a href="https://www.sunshinephotocart.com/pro/?utm_source=plugin&utm_medium=link&utm_campaign=about"><img src="<?php echo SUNSHINE_URL; ?>/assets/images/pro-upgrade.png" alt="Go Pro" /></a>
			</div>
		<?php } ?>

		<div class="sunshine-about-content">

			<?php if ( isset( $_GET['sunshine_install'] ) ) { ?>
				<div id="sunshine-get-started">
					<h2>Getting started is easy!</h2>
					<style>.embed-container { position: relative; padding-bottom: 56.25%; height: 0; overflow: hidden; max-width: 100%; } .embed-container iframe, .embed-container object, .embed-container embed { position: absolute; top: 0; left: 0; width: 100%; height: 100%; }</style><div class='embed-container'><iframe width="420" height="285" src="https://www.youtube.com/embed/t1IRNUASJSA?modestbranding=1&rel=0&showinfo=0" frameborder="0" allowfullscreen></iframe></div>
					<ol>
						<li><a href="http://www.facebook.com/sunshinephotocart" target="_blank">Follow us on Facebook</a> or <a href="http://eepurl.com/bzxukv" target="_blank">join our email newsletter</a> to stay updated with new features, important bug fixes, promos and more!</li>
						<li><a href="<?php echo admin_url( 'admin.php?page=sunshine_addons' ); ?>">Check out the add-ons</a> to get more advanced functionality</li>
						<li><a href="<?php echo admin_url( 'admin.php?page=sunshine' ); ?>">Configure your settings</a></li>
						<li><a href="<?php echo admin_url( 'edit.php?post_type=sunshine-product' ); ?>">Create your products</a></li>
						<li><a href="<?php echo admin_url( 'edit.php?post_type=sunshine-gallery' ); ?>">Create a gallery</a></li>
						<li>Invite your clients/users to view your galleries</li>
					</ol>
					<p><strong>Get more in depth help and how-to articles by going through the <a href="https://www.sunshinephotocart.com/docs/?utm_source=plugin&utm_medium=link&utm_campaign=docs">documentation</a></strong></p>
				</div>
			<?php } else { ?>

				<div class="sunshine-changelog">
					<?php
					$readme        = file_get_contents( SUNSHINE_PATH . '/readme.txt' );
					$readme_pieces = explode( '== Changelog ==', $readme );
					$changelog     = nl2br( htmlspecialchars( trim( $readme_pieces[1] ) ) );
					$changelog     = str_replace( array( ' =', '= ' ), array( '</h3>', '<h3>' ), $changelog );
					$nth = nth_strpos( $changelog, '<h3>', 7, true );
					if ( $nth !== false ) {
						$changelog = substr( $changelog, 0, $nth );
					}
					?>
					<h2><?php _e( 'Recent Improvements', 'sunshine' ); ?></h2>
					<div class="changelog"><?php echo $changelog; ?></div>
					<p><a href="https://wordpress.org/plugins/sunshine-photo-cart/#developers" target="_blank"><?php _e( 'See the full Changelog', 'sunshine' ); ?></a></p>
				</div>

			<?php } ?>

		</div>

	</div>
	<?php
}

function nth_strpos( $str, $substr, $n, $stri = false ) {
	if ( $stri ) {
		$str    = strtolower( $str );
		$substr = strtolower( $substr );
	}
	$ct  = 0;
	$pos = 0;
	while ( ( $pos = strpos( $str, $substr, $pos ) ) !== false ) {
		if ( ++$ct == $n ) {
			return $pos;
		}
		$pos++;
	}
	return false;
}

function sunshine_system_info() {
	global $sunshine;
	?>
<div class="wrap sunshine">
		<h2>System Information</h2>
		<p>Use the information below when submitting tickets or questions via <a href="http://www.sunshinephotocart.com/support" target="_blank">Sunshine Support</a>.</p>

<textarea id="sunshine-system-info" readonly="readonly" style="font-family: 'courier new', monospace; margin: 10px 0 0 0; width: 900px; height: 400px;" title="To copy the system info, click below then press Ctrl + C (PC) or Cmd + C (Mac).">

### Begin System Info ###

Home Page:                <?php echo site_url() . "\n"; ?>
Gallery URL:              <?php echo get_permalink( $sunshine->options['page'] ) . "\n"; ?>
Admin:                 	  <?php echo admin_url() . "\n"; ?>

WordPress Version:        <?php echo get_bloginfo( 'version' ) . "\n"; ?>

PHP Version:              <?php echo PHP_VERSION . "\n"; ?>
PHP Memory Limit:         <?php echo ini_get( 'memory_limit' ) . "\n"; ?>
WordPress Memory Limit:   <?php echo ( sunshine_let_to_num( WP_MEMORY_LIMIT ) / ( 1024 * 1024 ) ) . 'MB'; ?><?php echo "\n"; ?>
ImageMagick:              <?php
	echo ( extension_loaded( 'imagick' ) ) ? 'Yes' : 'No';
	echo  "\n";
	?>
Image Quality:            <?php echo apply_filters( 'jpeg_quality', 60 ); ?>
<?php do_action( 'sunshine_sunshine_info' ); ?>


ACTIVE PLUGINS:

<?php
$plugins        = get_plugins();
$active_plugins = get_option( 'active_plugins', array() );

foreach ( $plugins as $plugin_path => $plugin ) :

//If the plugin isn't active, don't show it.
if ( ! in_array( $plugin_path, $active_plugins ) ) {
	continue;
}
?>
<?php echo $plugin['Name']; ?>: <?php echo $plugin['Version']; ?>

<?php endforeach; ?>

CURRENT THEME:

<?php
if ( get_bloginfo( 'version' ) < '3.4' ) {
	$theme_data = get_theme_data( get_stylesheet_directory() . '/style.css' );
	echo $theme_data['Name'] . ': ' . $theme_data['Version'];
} else {
	$theme_data = wp_get_theme();
	echo $theme_data->Name . ': ' . $theme_data->Version;
}
?>


SUNSHINE SETTINGS:

<?php
foreach ( $sunshine->options as $key => $value ) :
	if ( strpos( 'token', $key ) !== false || strpos( 'key', $key ) !== false ) {
		continue; // Exclude some more sensitive items
	}
	$value = maybe_unserialize( $value );
	if ( is_array( $value ) ) {
		$values = $value;
		$value  = '';
		foreach ( $values as $k => $v ) {
			$value .= $k . ': ' . $v . '|';
		}
	}
	echo $key . ': ' . $value . "\r\n";
endforeach;
?>

IMAGE SIZES:

<?php
global $_wp_additional_image_sizes;
foreach ( $_wp_additional_image_sizes as $name => $image_size ) {
$crop = ( $image_size['crop'] ) ? 'cropped' : 'not cropped';
?>
<?php echo $name . ': ' . $image_size['width'] . 'x' . $image_size['height'] . ' (' . $crop . ')'; ?>

<?php } ?>

### End System Info ###
</textarea>

	</div>
	<p><button class="button button-primary" onclick="sunshine_copy_system_info()"><?php _e( 'Copy system info to clipboard', 'sunshine' ); ?></button></p>
	<script>
	function sunshine_copy_system_info() {
		  var copyText = document.getElementById( "sunshine-system-info" );
		  copyText.select();
		document.execCommand( "copy" );
	}
	</script>
	<p><?php _e( 'Our support team may ask you to manually run the update process.', 'sunshine' ); ?> <a href="admin.php?page=sunshine_system_info&amp;sunshine_force_update=1"><?php _e( 'Click here to do so', 'sunshine' ); ?></a></p>
	<?php

}

function sunshine_tools() {
	global $sunshine;
	?>
	<div class="wrap sunshine">
		<h2>Sunshine Tools</h2>
		<?php if ( !isset( $_GET['sunshine_regenerate_images'] ) ) { ?>
			<p>If you have changed thumbnail size, digital download size or watermark settings, you need to regenerate images.</p>
			<p><a href="<?php echo admin_url( 'admin.php?page=sunshine_tools&sunshine_regenerate_images' ); ?>" class="button button-primary">Regenerate Images</a></p>
		<?php
		} else {
			sunshine_regenerate_images();
		}
		?>

	</div>
<?php
}

function sunshine_regenerate_images() {
	global $wpdb;
	$gallery_id = ( isset( $_GET['sunshine_gallery'] ) ) ? intval( $_GET['sunshine_gallery'] ) : '';
	if ( isset( $_GET['sunshine_gallery'] ) ) {
		$image_sql = "SELECT COUNT(*) as total FROM {$wpdb->posts}
					  WHERE post_type = 'attachment' AND post_parent = {$gallery_id};";
	} else {
		$image_sql = "SELECT COUNT(*) as total FROM {$wpdb->posts}
					  WHERE post_type = 'attachment' AND post_parent IN (
						SELECT ID FROM {$wpdb->posts}
						WHERE post_type = 'sunshine-gallery'
					  );";
	}
	$count = $wpdb->get_row($image_sql)->total;

?>
	<h3>Regenerating Images...</h3>
	<div id="progress-bar" style="background: #000; height: 30px; position: relative;">
		<div id="percentage" style="height: 30px; background-color: green; width: 0%;"></div>
		<div id="processed" style="position: absolute; top: 0; left: 0; width: 100%; color: #FFF; text-align: center; font-size: 18px; height: 30px; line-height: 30px;">
			<span id="processed-count">0</span> of <span id="processed-total"><?php echo $count; ?></span>
		</div>
	</div>
	<p align="center" id="abort"><a href="<?php echo admin_url( 'admin.php?page=sunshine_tools' ); ?>"><?php _e( 'Abort', 'sunshine' ); ?></a></p>
	<ul id="results"></ul>
	<script type="text/javascript">
	jQuery(document).ready(function($) {
		var processed = 0;
		var total = <?php echo $count; ?>;
		var percent = 0;
		function sunshine_regenerate_image( item_number ) {
			var data = {
				'action': 'sunshine_regenerate_image',
				'gallery': '<?php echo $gallery_id; ?>',
				'item_number': item_number
			};
			$.postq('sunshineimageregenerate', ajaxurl, data, function(response) {
				var obj = $.parseJSON( response );
				processed++;
				if ( processed >= total ) {
					$('#abort').hide();
					$('#return').show();
				}
				$('#processed-count').html(processed);
				percent = Math.round( (processed / total) * 100);
				$('#percentage').css( 'width', percent+'%' );
				if ( obj.error ) {
					$( '#results' ).append( '<li><a href="post.php?action=edit&post=' + obj.image_id + '" style="color: #FF0000;">' + obj.file + '</a>: ' + obj.error + '</li>' );
				} else {
					$( '#results' ).append( '<li><a href="post.php?action=edit&post=' + obj.image_id + '">' + obj.file + '</a></li>' );
				}
			}).fail( function( jqXHR ) {
				if ( jqXHR.status == 500 || jqXHR.status == 0 ){
					$( '#results' ).append( '<li><strong><?php esc_js( __( 'Cannot process image, likely out of memory', 'sunshine' ) ); ?></strong></li>' );
				}
			});
		}
		for (i = 1; i <= total; i++) {
			sunshine_regenerate_image( i );
		}
	});
	</script>

<?php

}

add_action( 'wp_ajax_sunshine_regenerate_image', 'sunshine_regenerate_image' );
function sunshine_regenerate_image() {
	global $wpdb;

	set_time_limit( 600 );

	$item_number = intval( $_POST['item_number'] );
	$limit = $item_number - 1;

	if ( !empty( $_POST['gallery'] ) ) {
		$gallery_id = intval( $_POST['gallery'] );
		$image_sql = "SELECT * FROM {$wpdb->posts}
					  WHERE post_type = 'attachment' AND post_parent = {$gallery_id}
					  ORDER BY ID DESC
					  LIMIT {$limit}, 1;";
	} else {
		$image_sql = "SELECT * FROM {$wpdb->posts}
					  WHERE post_type = 'attachment' AND post_parent IN (
						SELECT ID FROM {$wpdb->posts}
						WHERE post_type = 'sunshine-gallery'
					  )
					  ORDER BY ID DESC
					  LIMIT {$limit}, 1;";
	}

	$image = $wpdb->get_row( $image_sql );

	if ( function_exists( 'wp_get_original_image_path' ) ) {
		$fullsizepath = wp_get_original_image_path( $image->ID );
	} else {
		$fullsizepath = get_attached_file( $image->ID );
	}
	if ( is_wp_error( $fullsizepath ) ) {
		echo json_encode( array( 'status' => 'error', 'file' => $image->post_name, 'image_id' => $image->ID, 'error' => __( 'Could not find original file to regenerate from', 'sunshine' ) ) );
		return;
	}

	// Delete all but orig file
	$file_info = pathinfo( $fullsizepath );
	foreach ( glob( $file_info['dirname'] . DIRECTORY_SEPARATOR . $file_info['filename'] . '-*.' . $file_info['extension'] ) as $file_size ) {
		wp_delete_file( $file_size );
	}

	$old_metadata = wp_get_attachment_metadata( $image->ID );
	$new_metadata = wp_generate_attachment_metadata( $image->ID, $fullsizepath );

	$wp_upload_dir = dirname( $fullsizepath ) . DIRECTORY_SEPARATOR;
	$image_sizes[] = 'sunshine-thumbnail';
	$image_sizes = apply_filters( 'sunshine_image_sizes', $image_sizes );
	foreach ( $old_metadata['sizes'] as $old_size => $old_size_data ) {
		if ( in_array( $old_size, $image_sizes ) ) {
			continue;
		}
		//wp_delete_file( $wp_upload_dir . $old_size_data['file'] );
		unset( $new_metadata['sizes'][ $old_size ] );
	}

	wp_update_attachment_metadata( $image->ID, $new_metadata );

	echo json_encode( array( 'status' => 'success', 'file' => $image->post_name, 'image_id' => $image->ID ) );

	exit;

}


function sunshine_addons() {
	global $sunshine;
	//if ( get_option( 'sunshine_pro_license_active') == 'valid' ) return;

	?>
	<div id="sunshine-header">
		<h1><?php _e( 'Add-ons for Sunshine Photo Cart', 'sunshine' ); ?></h1>
		<p>
			<?php _e( 'Go beyond the basics, Sunshine’s add-ons let you maximize your profits to help you build a more profitable client photo sales process.', 'sunshine' ); ?>
		</p>
	</div>

	<div class="wrap sunshine-wrap" id="sunshine-addons-wrap">

		<?php sunshine_promos(); ?>

	</div>
	<?php
}

//add_action( 'sunshine_options_header', 'sunshine_promos' );

function sunshine_promos() {
	global $sunshine;
	if ( $sunshine->is_pro() ) {
		return;
	}
	?>
	<div id="sunshine-promos">

		<div class="sunshine-upgrade">
			<a href="https://www.sunshinephotocart.com/pro/?utm_source=plugin&utm_medium=link&utm_campaign=about"><img src="<?php echo SUNSHINE_URL; ?>/assets/images/pro-upgrade.png" alt="Go Pro" /></a>
		</div>

		<?php
		$addons = get_option( 'sunshine_addons' );
		if ( $addons ) {
			$addons = (object) json_decode( $addons );
			echo '<ul id="sunshine-addons">';
			foreach ( $addons as $addon ) {
				if ( is_plugin_active( 'sunshine-' . $addon->slug . '/' . $addon->slug . '.php' ) ) {
					$action = '<p><span class="dashicons dashicons-yes"></span> Installed!</p>';
					$class  = 'sunshine-installed';
				} else {
					$action = '<p><span class="price">' . $addon->price . '</span> <a href="' . $addon->url . '?utm_source=plugin&utm_medium=link&utm_campaign=addons-list" target="_blank">' . __( 'Learn more & buy &raquo;', 'sunshine' ) . '</a></p>';
					$class  = 'available';
				}
				echo '<li class="' . $class . '"><h3><a href="' . $addon->url . '?utm_source=plugin&utm_medium=link&utm_campaign=addons-list" target="_blank">' . $addon->title . '</a></h3><p>' . $addon->excerpt . '</p>' . $action . '</li>';
			}
			echo '</ul>';
		}
		?>
		<p id="sunshine-disclaimer"><?php _e( 'All licenses are subject to annual renewal for continued updates and access to support', 'sunshine' ); ?></p>
	</div>
	<?php
}

add_filter( 'cron_schedules', 'sunshine_cron_add_weekly' );
function sunshine_cron_add_weekly( $schedules ) {
	// add a 'weekly' schedule to the existing set
	$schedules['weekly'] = array(
		'interval' => 604800,
		'display'  => __( 'Once Weekly' ),
	);
	return $schedules;
}

add_action( 'admin_init', 'sunshine_addon_check_cron' );
function sunshine_addon_check_cron() {
	if ( isset( $_GET['page'] ) && $_GET['page'] == 'sunshine_addons' ) {
		$plugins = get_option( 'sunshine_addons' );
		if ( empty( $plugins ) ) {
			sunshine_get_addons();
		}
	}
}

add_action( 'sunshine_addon_check', 'sunshine_get_addons' );
add_action( 'sunshine_install', 'sunshine_get_addons' );
function sunshine_get_addons() {
	global $sunshine;

	if ( $sunshine->is_pro() ) {
		return;
	}

	$url = SUNSHINE_STORE_URL . '/?sunshine_addons_feed';

	$feed = wp_remote_get( esc_url_raw( $url ), array( 'sslverify' => false ) );
	if ( ! is_wp_error( $feed ) ) {
		if ( isset( $feed['body'] ) && strlen( $feed['body'] ) > 0 ) {
			$addons = wp_remote_retrieve_body( $feed );
			update_option( 'sunshine_addons', $addons );
		}
	}
}

add_action( 'admin_init', 'sunshine_help' );
function sunshine_help() {
	if ( isset( $_GET['page'] ) && $_GET['page'] == 'sunshine_help' ) {
		wp_redirect( 'https://www.sunshinephotocart.com/support/?utm_source=plugin&utm_medium=link&utm_campaign=help' );
		exit;
	}
}

add_action( 'save_post', 'sunshine_flush_rewrite_page_save' );
function sunshine_flush_rewrite_page_save( $post_id ) {
	global $sunshine;
	if ( $post_id == $sunshine->options['page'] ) {
		flush_rewrite_rules();
	}
}

add_filter( 'admin_footer_text', 'sunshine_admin_footer_text' );
function sunshine_admin_footer_text( $footer_text ) {
	global $typenow;

	if ( $typenow == 'sunshine-gallery' || $typenow == 'sunshine-product' || $typenow == 'sunshine-order' || $typenow == 'sunshine-product' || isset( $_GET['page'] ) && strpos( $_GET['page'], 'sunshine' ) !== false ) {
		$rate_text = sprintf(
			__( 'Thank you for using <a href="%1$s" target="_blank">Sunshine Photo Cart</a>! Please <a href="%2$s" target="_blank">rate us</a> on <a href="%2$s" target="_blank">WordPress.org</a>', 'sunshine' ),
			'https://www.sunshinephotocart.com?utm_source=plugin&utm_medium=link&utm_campaign=rate',
			'https://wordpress.org/support/view/plugin-reviews/sunshine-photo-cart?filter=5#postform'
		);

		return str_replace( '</span>', '', $footer_text ) . ' | ' . $rate_text . '</span>';
	}

	return $footer_text;

}


/**********************
Clean up Media Library
***********************/
// For main Media Gallery page
add_filter( 'ajax_query_attachments_args', 'sunshine_clean_media_library' );
function sunshine_clean_media_library( $query ) {
	global $sunshine;
	if ( ! $sunshine->options['show_media_library'] ) {
		$args = array(
			'post_type'   => 'sunshine-gallery',
			'nopaging'    => true,
			'post_status' => 'publish,private,trash',
			'fields' => 'ids'
		);
		$gallery_ids = get_posts( $args );
		if ( !empty( $gallery_ids ) ) {
			$query['post_parent__not_in'] = $gallery_ids;
		}
	}
	return $query;
}

// For popup Media Gallery, this is done via AJAX call and has different filter
add_filter( 'ajax_query_attachments_args', 'sunshine_wp_ajax_query_attachments', 1 );
function sunshine_wp_ajax_query_attachments( $query ) {
	global $sunshine;
	$this_post = get_post( $_POST['post_id'] );
	if ( is_admin() && $this_post && $query && $query['post_type'] == 'attachment' && $this_post->post_type != 'sunshine-gallery' && empty( $sunshine->options['show_media_library'] ) ) {
		$args = array(
			'post_type' => 'sunshine-gallery',
			'nopaging' => true,
			'post_status' => 'any',
			'fields' => 'ids'
		);
		$gallery_ids = get_posts( $args );
		if ( !empty( $gallery_ids ) ) {
			$query['post_parent__not_in'] = $gallery_ids;
		}
	}
	return $query;
}

add_filter( 'pre_get_posts', 'sunshine_media_library_list' );
function sunshine_media_library_list( $query ) {
	global $sunshine;
	if ( $query->is_main_query() && empty( $sunshine->options['show_media_library'] ) ) {
		$screen = get_current_screen();
		if ( !empty( $screen ) && $screen->base == 'upload' ) {
			$args = array(
				'post_type' => 'sunshine-gallery',
				'nopaging' => true,
				'post_status' => 'any',
				'fields' => 'ids'
			);
			$gallery_ids = get_posts( $args );
			if ( !empty( $gallery_ids ) ) {
				$query->set( 'post_parent__not_in', $gallery_ids );
			}
		}
	}
}

add_action( 'admin_bar_menu', 'sunshine_admin_bar_view_client_galleries', 768 );
function sunshine_admin_bar_view_client_galleries() {
	global $wp_admin_bar, $sunshine;
	if ( is_admin() ) {
		$wp_admin_bar->add_node(
			array(
				'id'     => 'sunshine-client-galleries',
				'title'  => __( 'View Client Galleries', 'sunshine' ),
				'href'   => get_permalink( $sunshine->options['page'] ),
				'parent' => 'site-name',
			)
		);
	}
}

/**********************
Move images from pre 2.4
***********************/
add_action( 'admin_enqueue_scripts', 'sunshine_update_image_location_ajaxq' );
function sunshine_update_image_location_ajaxq( $hook ) {
	if ( isset( $_GET['page'] ) && ( $_GET['page'] == 'sunshine_update_image_location' || $_GET['page'] == 'sunshine_tools' ) ) {
		wp_enqueue_script( 'ajaxq', SUNSHINE_URL . 'assets/js/ajaxq.js' );
	}
}

add_action( 'admin_menu', 'sunshine_register_update_image_location_page' );
function sunshine_register_update_image_location_page() {
	add_submenu_page(
		null,
		'Update Image Location',
		'Update Image Location',
		'manage_options',
		'sunshine_update_image_location',
		'sunshine_update_image_location_page'
	);
}

function sunshine_update_image_location_page() {

	$start         = ( isset( $_GET['start'] ) ) ? $_GET['start'] : 0;
	$args          = array(
		'post_type'      => 'sunshine-gallery',
		'posts_per_page' => 1,
		'post_status'    => 'any',
		'offset'         => $start,
		'meta_query'     => array(
			array(
				'key'     => '24_update',
				'compare' => 'NOT EXISTS',
				'value'   => '',
			),
		),
	);
	$galleries     = get_posts( $args );
	$gallery_count = count( $galleries );

	if ( $gallery_count > 0 ) {
		$image_count = 0;
		$image_ids   = array();
		foreach ( $galleries as $gallery ) {
			$args        = array(
				'post_type'      => 'attachment',
				'posts_per_page' => -1,
				'post_status'    => 'any',
				'post_parent'    => $gallery->ID,
				'meta_query'     => array(
					array(
						'key'     => '24_update',
						'compare' => 'NOT EXISTS',
						'value'   => '',
					),
				),
			);
			$images      = get_posts( $args );
			$image_count = count( $images );
			foreach ( $images as $image ) {
				$image_ids[] = $image->ID;
			}
		}
	} else {
		delete_option( 'sunshine_update_image_location' );
	}
	$start++;
	$redirect_url = 'admin.php?page=sunshine_update_image_location&start=' . $start;
	?>

	<div class="wrap sunshine">
		<h2>Sunshine Upgrade Process</h2>
		<?php if ( $gallery_count > 0 ) { ?>
		<h3>Processing "<?php echo $galleries[0]->post_title; ?>"</h3>
		<p>If you have a lot of images, this could take a little while. <strong>Please do not leave this page until it is done.</strong></p>
		<div id="progress-bar" style="background: #000; height: 30px; position: relative;">
			<div id="percentage" style="height: 30px; background-color: green; width: 0%;"></div>
			<div id="processed" style="position: absolute; top: 0; left: 0; width: 100%; color: #FFF; text-align: center; font-size: 18px; height: 30px; line-height: 30px;">
				<span id="processed-count">0</span> of <span id="processed-total"><?php echo $image_count; ?></span>
			</div>
		</div>
		<ul id="errors"></ul>
		<script type="text/javascript">
		jQuery(document).ready(function($) {
			var images = [<?php echo esc_js( join( ',', $image_ids ) ); ?>];
			var processed = 0;
			var total = <?php echo $image_count; ?>;
			var percent = 0;
			var redirect_url = '<?php echo admin_url( $redirect_url ); ?>';
			if ( total == 0 ) {
				window.location.href = redirect_url;
			}
			function sunshine_update_image(image_id) {
				var data = {
					'action': 'sunshine_update_image_location',
					'image_id': image_id,
					'gallery_id': <?php echo $galleries[0]->ID; ?>
				};
				$.postq('imageupdatelocation', ajaxurl, data, function(response) {
					var obj = $.parseJSON( response );
					if ( obj.result == 'error' ) {
						$( '#errors' ).append( '<li><strong>' + obj.error + '</li>' );
						return;
					}
					processed++;
					if ( processed >= total ) {
						window.location.href = redirect_url;
					}
					$('#processed-count').html(processed);
					percent = Math.round( (processed / total) * 100);
					$('#percentage').css('width', percent+'%');
				});
			}
			for (i = 0; i < total; i++) {
				sunshine_update_image( images[i] );
			}
		});
		</script>
		<?php } else { ?>
			<p id="done" style="font-size: 18px; color: green;"><?php _e( 'Your galleries have been updated!', 'sunshine' ); ?></p>
		<?php } ?>
	</div>

	<?php
}

add_action( 'wp_ajax_sunshine_update_image_location', 'sunshine_update_image_location_ajax' );
function sunshine_update_image_location_ajax() {
	global $sunshine;
	if ( ! isset( $_POST['image_id'] ) || ! isset( $_POST['gallery_id'] ) ) {
		return;
	}
	$image_id   = intval( $_POST['image_id'] );
	$gallery_id = intval( $_POST['gallery_id'] );
	$upload_dir = wp_upload_dir();
	$meta       = wp_get_attachment_metadata( $image_id );

	$file_name = basename( $meta['file'] );

	if ( ! isset( $meta['file'] ) || ! file_exists( $upload_dir['basedir'] . '/' . $meta['file'] ) ) { // Meta data simply doesn't exist, let's just consider this updated and move on
		update_post_meta( $image_id, '24_update', 'yes' );
		echo json_encode(
			array(
				'result'  => 'success',
				'message' => __(
					'Image could not be found, just moving on',
					'sunshine'
				),
			)
		);
		exit;
	}

	// New folder path
	$new_path = $upload_dir['basedir'] . '/sunshine/' . $gallery_id;
	if ( ! is_dir( $new_path ) ) {
		$mkdir = mkdir( $new_path );
		if ( ! $mkdir ) {
			echo json_encode(
				array(
					'result'  => 'error',
					'message' => __(
						'Could not make new folder for gallery',
						'sunshine'
					),
				)
			);
			exit;
		}
	}
	// Move main image
	$rename = rename( $upload_dir['basedir'] . '/' . $meta['file'], $new_path . '/' . $file_name );
	if ( ! $rename ) {
		echo json_encode(
			array(
				'result'  => 'error',
				'message' => sprintf(
					__(
						'Could not move full size image: %s',
						'sunshine'
					),
					$meta['file']
				),
			)
		);
		exit;
	}

	// Move all sizes
	$folder = str_replace( basename( $meta['file'] ), '', $meta['file'] );
	foreach ( $meta['sizes'] as $size ) {
		if ( file_exists( $upload_dir['basedir'] . '/' . $folder . $size['file'] ) ) {
			$rename = rename( $upload_dir['basedir'] . '/' . $folder . $size['file'], $new_path . '/' . $size['file'] );
			if ( ! $rename ) {
				echo json_encode(
					array(
						'result'  => 'error',
						'message' => sprintf(
							__(
								'Could not move %s',
								'sunshine'
							),
							$size['file']
						),
					)
				);
				exit;
			}
		}
	}

	// Update meta data
	$new_meta         = $meta;
	$new_meta['file'] = 'sunshine/' . $gallery_id . '/' . $file_name;
	wp_update_attachment_metadata( $image_id, $new_meta );
	update_attached_file( $image_id, $new_path . '/' . $file_name );
	update_post_meta( $image_id, '24_update', 'yes' );

	echo json_encode(
		array(
			'result'  => 'success',
			'message' => sprintf(
				__(
					'Successfully moved %s',
					'sunshine'
				),
				$file_name
			),
		)
	);

	exit;
}


add_action( 'admin_notices', 'sunshine_order_admin_debug' );
function sunshine_order_admin_debug() {
	if ( isset( $_GET['post'] ) && isset( $_GET['sunshine_debug'] ) ) {
		echo '<div class="notice error">';
		$meta = get_post_meta( intval( $_GET['post'] ) );
		foreach ( $meta as $key => $values ) {
			echo '<strong>' . esc_html( $key ) . '</strong><br />';
			foreach ( $values as $value ) {
				$value = maybe_unserialize( $value );
				sunshine_dump_var( $value );
			}
		}
		echo '</div>';
	}
}

/* Customizations for Order Status admin */
add_action( 'admin_head', 'sunshine_order_status_admin_customizations' );
function sunshine_order_status_admin_customizations() {
	$screen = get_current_screen();
	if ( $screen->id == 'edit-sunshine-order-status' && isset( $_GET['tag_ID'] ) ) {
		if ( isset( $_GET['tag_ID'] ) ) {
			$current_status = get_term( intval( $_GET['tag_ID'] ) );
		}
		?>
	<script>
	jQuery( document ).ready( function($) {
		$( '.inline-edit-row label:nth-child(2)' ).remove();
		<?php
		if ( isset( $_GET['tag_ID'] ) ) {
			$core_statuses = sunshine_core_order_statuses();
			if ( in_array( $current_status->slug, $core_statuses ) ) {
				?>
			$( '#delete-link' ).remove();
			$( '.form-table tr:nth-child(2) p.description' ).html( '<?php echo esc_js( __( 'The slug for core order statuses is not editable as it could break Sunshine functionality', 'sunshine' ) ); ?>' );
			$( '.form-field input[name="slug"]' ).attr( 'disabled', 'disabled' );
				<?php
			}
		}
		?>
	});
	</script>
		<?php
	}
}

add_filter( 'sunshine-order-status_row_actions', 'sunshine_order_status_row_actions', 10, 2 );
function sunshine_order_status_row_actions( $actions, $term ) {
	$core_statuses = sunshine_core_order_statuses();
	if ( in_array( $term->slug, $core_statuses ) ) {
		unset( $actions['delete'] );
	}
	return $actions;
}


/* GDPR */
function sunshine_personal_data_exporter( $email_address, $page = 1 ) {

	$number = 500; // Limit us to avoid timing out
	$page   = (int) $page;

	$export_items = array();

	$orders = get_posts(
		array(
			'post_type' => 'sunshine-order',
			'number'    => $number,
			'paged'     => $page,
		)
	);

	foreach ( (array) $orders as $order ) {

		$item_id     = "sunshine-order-{$order->ID}";
		$group_id    = 'sunshine-order';
		$group_label = __( 'Orders', 'sunshine' );

		$order_data = sunshine_get_order_data( $order->ID );

		// Plugins can add as many items in the item data array as they want
		$data = array(
			array(
				'name'  => __( 'Order ID' ),
				'value' => $order->ID,
			),
			array(
				'name'  => __( 'Email' ),
				'value' => $order_data['email'],
			),
			array(
				'name'  => __( 'Phone' ),
				'value' => $order_data['phone'],
			),
			array(
				'name'  => __( 'Billing First Name' ),
				'value' => $order_data['first_name'],
			),
			array(
				'name'  => __( 'Billing Last Name' ),
				'value' => $order_data['last_name'],
			),
			array(
				'name'  => __( 'Billing Address' ),
				'value' => $order_data['address'],
			),
			array(
				'name'  => __( 'Billing Address 2' ),
				'value' => $order_data['address2'],
			),
			array(
				'name'  => __( 'Billing City' ),
				'value' => $order_data['city'],
			),
			array(
				'name'  => __( 'Billing State' ),
				'value' => $order_data['state'],
			),
			array(
				'name'  => __( 'Billing Country' ),
				'value' => $order_data['country'],
			),
			array(
				'name'  => __( 'Shipping First Name' ),
				'value' => $order_data['shipping_first_name'],
			),
			array(
				'name'  => __( 'Shipping Last Name' ),
				'value' => $order_data['shipping_last_name'],
			),
			array(
				'name'  => __( 'Shipping Address' ),
				'value' => $order_data['shipping_address'],
			),
			array(
				'name'  => __( 'Shipping Address 2' ),
				'value' => $order_data['shipping_address2'],
			),
			array(
				'name'  => __( 'Shipping City' ),
				'value' => $order_data['shipping_city'],
			),
			array(
				'name'  => __( 'Shipping State' ),
				'value' => $order_data['shipping_state'],
			),
			array(
				'name'  => __( 'Shipping Country' ),
				'value' => $order_data['shipping_country'],
			),
			array(
				'name'  => __( 'Notes' ),
				'value' => $order_data['notes'],
			),
		);

		$export_items[] = array(
			'group_id'    => $group_id,
			'group_label' => $group_label,
			'item_id'     => $item_id,
			'data'        => $data,
		);
	}

	// Tell core if we have more comments to work on still
	$done = count( $orders ) < $number;

	return array(
		'data' => $export_items,
		'done' => $done,
	);
}

add_filter( 'wp_privacy_personal_data_exporters', 'register_sunshine_personal_data_exporter', 10 );
function register_sunshine_personal_data_exporter( $exporters ) {
	$exporters[] = array(
		'exporter_friendly_name' => __( 'Comment Location Plugin' ),
		'callback'               => 'sunshine_personal_data_exporter',
	);
	return $exporters;
}

add_filter( 'wp_privacy_personal_data_erasers', 'register_sunshine_privacy_data_eraser', 94 );
function register_sunshine_privacy_data_eraser( $erasers ) {
	$erasers['sunshine-photo-cart'] = array(
		'eraser_friendly_name' => __( 'Sunshine Photo Cart', 'sunshine' ),
		'callback'             => 'sunshine_privacy_data_eraser',
	);
	return $erasers;
}

function sunshine_privacy_data_eraser( $email_address, $page = 1 ) {
	global $wpdb;

	$user = get_user_by( 'email', $email_address );

	$user_meta_removed = $wpdb->query( "DELETE FROM {$wpdb->usermeta} WHERE user_id = '{$user->ID}' AND meta_key LIKE 'sunshine%';" );

	$orders_removed = $wpdb->query( "DELETE FROM {$wpdb->posts} WHERE post_author = '{$user->ID}' AND post_type = 'sunshine-order';" );
	if ( $orders_removed ) {
		$wpdb->query( "DELETE meta FROM {$wpdb->postmeta} meta LEFT JOIN {$wpdb->posts} posts ON posts.ID = meta.post_id WHERE posts.ID IS NULL;" );
	}

	return array(
		'items_removed'  => ( $user_meta_removed || $orders_removed ) ? true : false,
		'items_retained' => false, // always false in this example
		'messages'       => array(), // no messages in this example
		'done'           => true,
	);
}

add_filter( 'plugin_action_links_sunshine-photo-cart/sunshine-photo-cart.php', 'sunshine_plugin_action_links' );
function sunshine_plugin_action_links( $links ) {
	global $sunshine;
	$settings_page = '<a href="' . admin_url( 'admin.php?page=sunshine' ) . '">' . __( 'Settings', 'sunshine' ) . '</a>';
	array_unshift( $links, $settings_page );
	if ( ! $sunshine->is_pro() ) {
		$upgrade_page = '<a href="https://www.sunshinephotocart.com/pro/?utm_source=plugin&utm_medium=link&utm_campaign=plugin_action_link" target="_blank"><b style="color: orange;">' . __( 'Go Pro', 'sunshine' ) . '</b></a>';
		array_unshift( $links, $upgrade_page );
	}
	return $links;
}


/**
  * Add survey to plugin deactivation
  *
  * @author Samuel Elh <samelh.com/contact>
  * @see https://samelh.com/blog
  */
add_action( 'admin_init', 'sunshine_plugin_deactivation_survey' );
function sunshine_plugin_deactivation_survey() {

	$plugin = 'sunshine-photo-cart/sunshine-photo-cart.php';

	// making sure we're running in plugins.php admin page
	if ( ! isset( $GLOBALS['pagenow'] ) || 'plugins.php' != $GLOBALS['pagenow'] ) {
		return;
	}

	// deactivations only
	if ( ! isset( $_REQUEST['action'] ) || 'deactivate' != $_REQUEST['action'] ) {
		return;
	}

	// run on our plugin deactivation
	if ( ! isset( $_REQUEST['plugin'] ) || $plugin != $_REQUEST['plugin'] ) {
		return;
	}

	// make sure we don't make plugin deactivations impossible
	if ( isset( $_REQUEST['ignore_sv'] ) ) {
		return;
	}

	if ( isset( $_POST['submit_sv'] ) ) {
		if ( isset( $_POST['reason'] ) ) {
			if ( $_POST['allow_contact'] == 'true' && is_email( $_POST['email'] ) ) {
				wp_mail(
					'derek@sunshinephotocart.com',
					'Deactivation Survey',
					sprintf(
						'%s: %s',
						sanitize_text_field( $_POST['reason'] ),
						sanitize_text_field( $_POST['explanation'] )
					),
					array( 'From: ' . get_bloginfo( 'name' ) . ' <' . sanitize_text_field( $_POST['email'] ) . '>' )
				);
			}

			$data = array(
				'version'     => SUNSHINE_VERSION,
				'reason'      => sanitize_text_field( $_POST['reason'] ),
				'explanation' => sanitize_text_field( $_POST['explanation'] ),
			);
			$args = array(
				'body'      => $data,
				'blocking'  => false,
				'sslverify' => false,
			);
			wp_remote_post( 'https://www.sunshinephotocart.com/?deactivation_survey=1', $args );
		}
		// refresh the page to continue to plugin deactivation
		wp_safe_redirect( add_query_arg( 'ignore_sv', 1 ) );
		exit;
	} else {
		ob_start();
		?>

		<style>
			body { position: relative; background: #FFFFFF; }
			label { display: block; margin: 0 0 5px 0; }
			.explain { display: none; margin: 3px 0 10px 20px; font-size: 13px; color: #666; font-style: italic; }
			.explain input { padding: 5px 8px; font-size: 13px; width: 50%; }
		</style>
		<a href="<?php echo admin_url( 'plugins.php' ); ?>" style="color: red; display: block; margin: -75px -30px 0 0; float: right; font-size: 12px; text-transform: uppercase; text-decoration: none;"><span style="display: inline-block; background: red; color: #FFF; font-size: 12px; width: 16px; height: 16px; line-height: 16px; text-align: center; font-weight: bold; border-radius: 50%;">×</span> Cancel</a>
		<h2>If you have a moment, please let me know why you are deactivating Sunshine Photo Cart:</h2>
		<form method="post">
			<label><input type="radio" name="reason" value="Could not understand" data-explain="couldnt-understand"> I couldn't understand how to make it work</label>
			<div id="couldnt-understand" class="explain">
				Sunshine has a <a href="https://www.youtube.com/watch?v=f8fLSHbpqwU" target="_blank">How To video</a>, <a href="https://www.sunshinephotocart.com/docs" target="_blank">Documentation</a>, and <a href="https://www.sunshinephotocart.com/support" target="_blank">Support options</a> to consider
			</div>
			<label><input type="radio" name="reason" value="Better plugin" data-explain="better-plugin"> Found a better plugin</label>
			<div id="better-plugin" class="explain">
				<input type="text" name="better-plugin-explanation" placeholder="Which plugin?" />
			</div>
			<label><input type="radio" name="reason" value="Missing feature" data-explain="missing-feature"> Sunshine is great, but it doesn't have a feature I need</label>
			<div id="missing-feature" class="explain">
				<input type="text" name="missing-feature-explanation" placeholder="Specific feature?" /><br />
				Did you know Sunshine has <a href="https://www.sunshinephotocart.com/addons" target="_blank">many add-ons</a>?
			</div>
			<label><input type="radio" name="reason" value="Not what I'm looking for"> It's not what I was looking for</label>
			<label><input type="radio" name="reason" value="Not working as expected" data-explain="not-working"> The plugin didn't work as expected</label>
			<div id="not-working" class="explain">
				<input type="text" name="not-working-explanation" placeholder="What was not working for you?" />
			</div>
			<label><input type="radio" name="reason" value="Testing"> Deactivating temporarily to test something</label>
			<label><input type="radio" name="reason" value="Other" data-explain="other"> Other</label>
			<div id="other" class="explain">
				<input type="text" name="other-explanation" placeholder="Any additional details?" />
			</div>

			<p>
				<label><input type="checkbox" name="allow_contact" value="true" /> You can reach me at <input type="email" name="email" value="<?php bloginfo( 'admin_email' ); ?>" style="border: 0; border-bottom: 1px solid #999; padding: 0 0 3px 0; font-size: 13px; width: 200px;" /> to discuss in more detail</label>
			</p>

			<p>
				<input type="submit" name="submit_sv" value="Submit Survey and Deactivate Plugin" class="button button-large" />
				&nbsp;
				<a href="<?php echo add_query_arg( 'ignore_sv', 1 ); ?>" style="display: inline-block; vertical-align: middle; margin-left: 20px; font-size: 14px;">I just want to deactivate</a>
			</p>

			<p style="font-size: 13px; color: #666; font-style: italic;">The only information stored is your response to this survey and version of Sunshine. No additional data is collected about you or your website. You will not be added to any email list.</p>
		</form>

		<script>
		function myfunction(event) {
			console.log( event.target );
			console.log( event.target.dataset.explain );
			var divsToHide = document.getElementsByClassName("explain");
			for(var i = 0; i < divsToHide.length; i++){
				divsToHide[i].style.display = "none";
			}
			document.getElementById( event.target.dataset.explain ).style.display = "block";

		}
		document.querySelectorAll("input[name='reason']").forEach((input) => {
			input.addEventListener('change', myfunction);
		});

		/*
			jQuery( 'input[name="reason"]' ).change( function() {
				var reason = jQuery( 'input[name="reason"]:checked' ).data( 'explain' );
				jQuery( '.explain' ).hide();
				jQuery( '.explain input' ).attr( 'name', reason + '-explain' );
				jQuery( '#' + reason ).show();
				jQuery( '#' + reason + ' input' ).attr( 'name', 'explanation' );
			});
			*/
		</script>

		<?php
		wp_die( ob_get_clean(), 'Deactivating Sunshine Photo Cart...' );
	}
}

add_action( 'admin_menu', function() {
    add_submenu_page(
        null,
		'Sunshine Upgrade to 2.8.26',
        'Sunshine Upgrade to 2.8.26',
        'manage_options',
        'sunshine_upgrade_2826',
        'sunshine_upgrade_2826'
    );
	add_submenu_page(
        null,
		'Sunshine Upgrade to 2.8.33',
        'Sunshine Upgrade to 2.8.33',
        'manage_options',
        'sunshine_upgrade_2833',
        'sunshine_upgrade_2833'
    );
} );

function sunshine_upgrade_2826() {
	global $sunshine;

	echo '<div class="wrap">';
	echo '<h1>Sunshine 2.8.26 Update Process</h1>';
	    // Your content
	if ( !isset( $_GET['action'] ) ) {
		echo '<script>
			jQuery( document ).ready(function(){
				jQuery.ajax({
					type: "POST",
					url: ajaxurl,
					timeout: 30000,
					data: {
						action: "sunshine_upgrade_2826_process_option1"
					},
					success: function(data) {
						jQuery( "#upgrade-processing" ).hide();
						jQuery( "#upgrade-success" ).show();
					},
					error: function( jqXHR, textStatus, errorThrown ) {
						jQuery( "#upgrade-processing" ).hide();
						jQuery( "#upgrade-fail" ).show();
					},
					statusCode: {
						500: function() {
							jQuery( "#upgrade-processing" ).hide();
							jQuery( "#upgrade-fail" ).show();
						}
					}
				});
			});
		</script>';
		echo '<div id="upgrade-processing"><img src="' . admin_url( 'images/spinner-2x.gif' ) . '" width="20" alt="Processing" style="display: inline-block; vertical-align: middle;" /> Attempting to automatically process update...</div>';
		echo '<div id="upgrade-success" style="display: none; font-weight: bold;">Success!</div>';
		echo '<div id="upgrade-fail" style="display: none;">';
			echo '<p><strong>Sorry, the simple process did not work for you...</strong></p>';
			echo '<p>So here is the deal: Sunshine 2.8.24 had a typo that created a new cron job entry in your database on every page load. 2.8.25 tried to delete it but failed in some instances where there was just too much data. Sunshine just tried again another way and failed.</p>';
			echo '<p>Now, we need to be a little more aggressive to fix the issue but it requires some additional steps. These additional steps should only take 1-2 minutes of your time.</p>';
			echo '<p>This option quickly deletes ALL crons, even ones created by other plugins though. After running this process you will need to <strong>deactivate and activate all plugins</strong> so they can setup their needed cron jobs again.</p>';
			echo '<p><a href="' . admin_url( '?page=sunshine_upgrade_2826&action=option2' ) . '" class="button">Click here to attempt Option 2</a><p>';
			echo '<p><strong>What is a cron and what exactly happened here?</strong></p>';
			echo '<p>A cron is what allows plugins to automatically run code at specified time. For example, Sunshine saves information that is in a user\'s cart to the database. After 30 days, Sunshine attempts to clear out that data since we assume it is no longer needed as the customer is not going to finish their purchase. This helps ensure the database does not get bloated or too large.</p>';
			echo '<p>Plugins like Sunshine are supposed to tell WordPress only one time to setup a cron schedule. Unfortunately, Sunshine accidentally was telling WordPress to do this on <em>every single page load</em> which caused some databases to get really big, really fast.</p>';
			echo '<p>The automated process tried to remove <em>only</em> the extra entries related to Sunshine. For some sites where there was just so much data it could not do this before the server failed. So Option 2 deletes ALL cron data including that of other plugins. Deactivating and reactivating your plugins allows those other plugins to then re-establish their needed crons.</p>';
			echo '<p><em>I am truly sorry for this inconvenience!<br />Derek Ashauer, Lead Developer</em></p>';
		echo '</div>';
	} elseif ( $_GET['action'] == 'option2' ) {
		delete_option( 'cron' );
		$sunshine->notices->delete_notice( 'upgrade_2826' );
		echo '<p><strong>All set!</strong></p>';
		echo '<p>Remember, you now need to <a href="' . admin_url( 'plugins.php' ) . '">deactivate your plugins</a> and then reactivate them so each plugin can re-establish necessary crons</p>';
	}

	echo '</div>';

	echo '<script>
		jQuery( document ).ready(function(){
			jQuery( ".sunshine-notice" ).remove();
		});
		</script>';

}

add_action( 'wp_ajax_sunshine_upgrade_2826_process_option1', 'sunshine_upgrade_2826_process_option1' );
function sunshine_upgrade_2826_process_option1() {
	global $sunshine;
	$result = wp_clear_scheduled_hook( 'sunshine_session_garbage_collection' );
	if ( $result === false ) {
		wp_die( 'FAIL', 'Upgrade test', array( 'response' => 500 ) );
		exit;
	}
	wp_schedule_event( current_time( 'timestamp' ), 'daily', 'sunshine_session_garbage_collection' );
	$sunshine->notices->delete_notice( 'upgrade_2826' );
	die();
}

/*
add_action( 'admin_enqueue_scripts', 'sunshine_enqueue_upgrade_2833_ajaxq' );
function sunshine_enqueue_upgrade_2833_ajaxq( $hook ) {
    if ( isset( $_GET['page'] ) && $_GET['page'] == 'sunshine_upgrade_2833' ) {
	    wp_enqueue_script( 'ajaxq', SUNSHINE_URL . 'assets/js/ajaxq.js' );
    }
}

function sunshine_upgrade_2833() {
	global $sunshine;
	$sunshine->notices->delete_notice( 'upgrade_2833' );
	$order_counts = wp_count_posts( 'sunshine-order' );
	$count = $order_counts->publish;
?>
<div class="wrap">
	<h1>Sunshine 2.8.33 Update Process</h1>
	<p>Sunshine had some math errors when calculating tax amounts when prices have tax included. This update process is going through all your orders and updating the tax amount saved for each order.</p>

	<div id="progress-bar" style="background: #000; height: 30px; position: relative;">
		<div id="percentage" style="height: 30px; background-color: green; width: 0%;"></div>
		<div id="processed" style="position: absolute; top: 0; left: 0; width: 100%; color: #FFF; text-align: center; font-size: 18px; height: 30px; line-height: 30px;">
			<span id="processed-count">0</span> of <span id="processed-total"><?php echo $count; ?></span>
		</div>
	</div>
	<p align="center" id="return" style="display: none;">Complete!</p>

	<script>
		jQuery( document ).ready(function($){
			$( '.sunshine-notice' ).remove();

			var processed = 0;
			var total = <?php echo $count; ?>;
			var percent = 0;

			function sunshine_upgrade_2833_processor( item_number ) {
				var data = {
					'action': 'sunshine_upgrade_2833_process',
					'item_number': item_number
				};
				$.postq( 'sunshineupgradeprocessor', ajaxurl, data, function(response) {
					var obj = $.parseJSON( response );
					processed++;
					if ( processed >= total ) {
						$('#return').show();
					}
					$('#processed-count').html(processed);
					percent = Math.round( (processed / total) * 100);
					$('#percentage').css( 'width', percent+'%' );
					if ( obj.error ) {
						$( '#errors' ).append( '<li><strong>' + obj.file + '</strong> - ' + obj.error + '</li>' );
					}
				}).fail( function( jqXHR ) {
                    if ( jqXHR.status == 500 || jqXHR.status == 0 ){
                        $( '#errors' ).append( '<li><strong><?php esc_js( __( 'Cannot process order', 'sunshine' ) ); ?></strong></li>' );
                    }
                });
			}
			for (i = 1; i <= total; i++) {
				console.log( i );
			    sunshine_upgrade_2833_processor( i );
			}

		});
	</script>
	</div>

<?php
}

add_action( 'wp_ajax_sunshine_upgrade_2833_process', 'sunshine_upgrade_2833_process' );
function sunshine_upgrade_2833_process() {
	global $sunshine;
	// Get order and process
	//sunshine_log( $_POST );

	$args = array(
		'post_type' => 'sunshine-order',
		'posts_per_page' => 1,
		'offset' => intval( $_POST['item_number'] ) - 1
	);
	$orders = get_posts( $args );
	if ( !empty( $orders ) ) {
		$order = $orders[0];
		$order_data = maybe_unserialize( get_post_meta( $order->ID, '_sunshine_order_data', true ) );
		( $order_data );
	}

	echo json_encode( array( 'status' => 'success', 'order_id' => 'g' ) );

	die();
}
*/
